@@ -1,11 +1,11 @@
-from typing import List, Type
+from typing import List, Type, Iterable, Tuple
 from pydfs_lineup_optimizer.player import LineupPlayer
 from pydfs_lineup_optimizer.lineup_printer import BaseLineupPrinter, LineupPrinter
 
 
 class Lineup:
-    def __init__(self, players: List[LineupPlayer], printer: Type[BaseLineupPrinter] = LineupPrinter):
-        self.players = players
+    def __init__(self, players: Iterable[LineupPlayer], printer: Type[BaseLineupPrinter] = LineupPrinter):
+        self.players = tuple(players)
         self.printer = printer()
 
     def __iter__(self):
@@ -21,13 +21,18 @@ class Lineup:
         return 'Lineup: projection %s, budget %s' % (self.fantasy_points_projection, self.salary_costs)
 
     def __hash__(self):
-        return hash(sorted(p.id for p in self))
+        return hash(self.__key())
 
     def __eq__(self, other):
-        return hash(self) == hash(other)
+        if isinstance(other, Lineup):
+            return self.__key() == other.__key()
+        return NotImplemented
+
+    def __key(self) -> Tuple[str, ...]:
+        return tuple(sorted(p.id for p in self))
 
     @property
-    def lineup(self) -> List[LineupPlayer]:
+    def lineup(self) -> Iterable[LineupPlayer]:
         return self.players
 
     @property
