@@ -2,6 +2,7 @@ from pydfs_lineup_optimizer.settings import BaseSettings, LineupPosition
 from pydfs_lineup_optimizer.constants import Sport, Site
 from pydfs_lineup_optimizer.sites.sites_registry import SitesRegistry
 from pydfs_lineup_optimizer.sites.yahoo.importer import YahooCSVImporter
+from pydfs_lineup_optimizer.lineup_exporter import YahooCSVLineupExporter
 
 
 class YahooSettings(BaseSettings):
@@ -9,6 +10,7 @@ class YahooSettings(BaseSettings):
     budget = 200
     max_from_one_team = 6
     csv_importer = YahooCSVImporter
+    csv_exporter = YahooCSVLineupExporter
 
 
 @SitesRegistry.register_settings
